function love.load()
	math.randomseed(os.time())
	
  local pxCode = [[
    extern number time;
    number t;
    vec4 effect(vec4 color, Image tex, vec2 tc, vec2 pc)
    {
      t = time * 1.5;
      color = Texel(tex, tc);
      return vec4(vec3(sin(t + 5)+0.3, -sin(t+5)+0.3, sin(t + 10)) * (max(color.r, max(color.g, color.b))), 1.0);
    }
  ]]
  local vxCode = [[
    varying vec4 vpos;

    vec4 position( mat4 transform_projection, vec4 vertex_position )
    {
        vpos = vertex_position;
        return transform_projection * vertex_position;
    }
  ]]
  
  bgShader = love.graphics.newShader(pxCode,vxCode)
  t = 0
  
  --create grey background gradient (we'll tint it with shaders ;) )
	background = gradient { { 150, 150, 150 }, { 255, 255, 255 }, { 150, 150, 150 } }
	backy1 = 0 --top co-ords of the first background instance
	backy2 = love.window.getHeight() --top co-ords of the second (sadly we have to track both)
	backspeed = 200
end

function love.update(dt)
	--shader updates
  t = t + math.min(dt, 1/30)
  bgShader:send("time", t)
  
  --update backgrounds pos
	local screenh = love.window.getHeight()
	backy1 = backy1 - backspeed * dt
	backy2 = backy1 + screenh
	if backy1 + screenh <= 0 then
		local dy = backy1 + screenh*2
		backy1 = screenh - dy
	elseif backy1 >= screenh then
		local dy = backy1 - screenh*2
		backy1 = dy
	end
	if backy2 + screenh <= 0 then
		local dy = backy2 + screenh*2
		backy2 = screenh - dy
	elseif backy2 >= screenh then
		local dy = backy2 - screenh*2
		backy2 = dy
	end
end

function love.draw()
	--draw background
	love.graphics.setShader(bgShader)
	drawinrect(background, 0, backy1, love.window.getWidth(), love.window.getHeight())
	drawinrect(background, 0, backy1+love.window.getHeight(), love.window.getWidth(), love.window.getHeight())
	
	love.graphics.setShader()
end

function love.keyreleased(key,code)
	if key == 'escape' then
		love.event.quit()
	end
end

function gradient(colors)
    local direction = colors.direction or "horizontal"
    if direction == "horizontal" then
        direction = true
    elseif direction == "vertical" then
        direction = false
    else
        error("Invalid direction '" .. tostring(direction) "' for gradient.  Horizontal or vertical expected.")
    end
    local result = love.image.newImageData(direction and 1 or #colors, direction and #colors or 1)
    for i, color in ipairs(colors) do
        local x, y
        if direction then
            x, y = 0, i - 1
        else
            x, y = i - 1, 0
        end
        result:setPixel(x, y, color[1], color[2], color[3], color[4] or 255)
    end
    result = love.graphics.newImage(result)
    result:setFilter('linear', 'linear')
    return result
end

function drawinrect(img, x, y, w, h, r, ox, oy, kx, ky)
    return -- tail call for a little extra bit of efficiency
    love.graphics.draw(img, x, y, r, w / img:getWidth(), h / img:getHeight(), ox, oy, kx, ky)
end